

#ifndef __I_JOYSTICK_H_INCLUDED__
#define __I_JOYSTICK_H_INCLUDED__

#include "irrTypes.h"
#include "irrString.h"
#include "IEventReceiver.h"
//#include "Keycodes.h"

namespace irr
{






////////////////////////////////////////////////////////////
/// \brief Structure holding a joystick's capabilities
///
////////////////////////////////////////////////////////////
struct SJoystickCapabilities
{
    SJoystickCapabilities() : PovHat(POV_HAT_UNKNOWN),ButtonCount(0)
    {
         //= 0;
        //std::fill(Axes, Axes + Joystick::AxisCount, false);
    }

    E_JOYSTICK_POVHAT PovHat;   	//!< An indication of whether the joystick has a POV hat.
    u32 ButtonCount;               ///< Number of buttons supported by the joystick
    u32 AxisCount;

    static const u8 MaximumNumberOfSupportedButtons = 32;
    static const u8 MaximumNumberOfSupportedAxis = 8;
};




struct SJoystickState {

    SJoystickState() : Buttons(0) {}
//, Axes(0)
	//! The number of buttons that the joystick has.
	// il vaut mieux un tableau
	//bool *Buttons;
	u32				Buttons;


	//! The number of axes that the joystick has, i.e. X, Y, Z, R, U, V.
	/** Note: with a Linux device, the POV hat (if any) will use two axes. These
	 *  will be included in this count. */
	u32				Axis[NUMBER_OF_AXES];

	u16 POV;


    bool isButtonPressed(int const& index) {
        return ( ( Buttons >> index) & 1) ;
    };
};



////////////////////////////////////////////////////////////
/// \brief Give access to the real-time state of the keyboard
///
////////////////////////////////////////////////////////////


// TODO le faire deriver de ReferenceCounted ?
// TODO should be a CJoystick in fact !
class IJoystick
{

protected:
// Should remain read-only
    const SJoystickCapabilities Capabilities;
    SJoystickState State;
public :



    // TODO should depend on joystick
    // Should be a SJoystickCaps ?


    const u8 Index;
	core::stringc	Name;   //!< The name that the joystick uses to identify itself.
    int Threshold;          //!< Threshold
	//u32				Buttons;    //!< Bit field
	//u32				Axes;
    //bool         Axes[Joystick::AxisCount]; ///< Support for each axis


//,Buttons(0),Axes(0)
    IJoystick(const char* name,const u8& JoyId,const SJoystickCapabilities& capabilities) :
                                Name(name), //"Unkown"
                                Index(JoyId),
                                Capabilities(capabilities),
                                Threshold(0)
    {
        // cstor
    };

    virtual ~IJoystick() {};


    /**
    **/
    static bool isConnected(u8 const& joyIndex);

    const SJoystickState& getState() const;
    //const SJoystickState&
    virtual void update() = 0;

    const SJoystickCapabilities& getCapabilities() { return Capabilities;};

    //static bool getCapabilities(const u8& joystickId,SJoystickCapabilities& caps);


    ////////////////////////////////////////////////////////////
    /// \brief Check if a key is pressed
    ///
    /// \param key Key to check
    ///
    /// \return True if the key is pressed, false otherwise
    ///
    ////////////////////////////////////////////////////////////
    bool isButtonPressed(int const& index) {
        return State.isButtonPressed(index);
    };

    // TODO identify axis as an enum ?
    //virtual bool hasAxis(int const& axis) = 0;
    //virtual s32 getAxisValue(E_JOYSTICK_AXIS const& axis) = 0;

//  Rely on SJoyctickCapabilites
//    virtual bool hasPOV() = 0;
//    virtual int getPOV() = 0;

    //void setThreshold();
};

} // namespace irr


#endif



